

namespace Esb.Math
{
	/// <summary>
	/// Integer Routines for ESBPCS.
	/// </summary>
#if (DEBUG)

	public class IntOps
#else
	sealed public class IntOps
#endif
	{

		/// <summary>
		/// Dummy Constructor - Class only contains static Methods
		/// </summary>
		private IntOps()
		{
		}

		#region Odd Methods

		/// <overloads>
		/// Determines whether an Integral value is Odd.
		/// </overloads>
		/// <summary>
		/// Determines whether an Integral value is Odd.
		/// </summary>
		/// <param name="x">Value to Test.</param>
		/// <returns>True if Odd, False if Even.</returns>
		public static bool Odd (int x)
		{
			return ((x % 2 == 1) || (x % 2 == -1));
		}

		/// <summary>
		/// Determines whether an Integral value is Odd.
		/// </summary>
		/// <param name="x">Value to Test.</param>
		/// <returns>True if Odd, False if Even.</returns>
		public static bool Odd (long x)
		{
			return ((x % 2 == 1) || (x % 2 == -1));
		}

		#endregion //Odd Methods

		#region Floor Methods

		/// <overloads>
		/// <para>Returns the Integer Value that is less than or equal to the specified number.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// <para>Returns the Integer Value that is less than or equal to the specified number.</para>
		/// <para>For "larger" values <see cref="Floor64"/></para>
		/// </summary>
		/// <param name="x">Value to be processed</param>
		/// <returns>Integer Value that is less than or equal to the specified number.</returns>
		public static int Floor (double x)
		{
			return (int) System.Math.Floor (x);
		}

		/// <summary>
		/// <para>Returns the Integer Value that is less than or equal to the specified number.</para>
		/// <para>For "larger" values <see cref="Floor64"/></para>
		/// </summary>
		/// <param name="x">Value to be processed</param>
		/// <returns>Integer Value that is less than or equal to the specified number.</returns>
		public static int Floor (decimal x)
		{
			return (int) System.Decimal.Floor (x);
		}

		/// <overloads>
		/// <para>Returns the Integer Value that is less than or equal to the specified number.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// Returns the Integer Value that is less than or equal to the specified number.
		/// </summary>
		/// <param name="x">Value to be processed</param>
		/// <returns>Integer Value that is less than or equal to the specified number.</returns>
		public static long Floor64 (double x)
		{
			return (long) System.Math.Floor (x);
		}

		/// <summary>
		/// Returns the Integer Value that is less than or equal to the specified number.
		/// </summary>
		/// <param name="x">Value to be processed</param>
		/// <returns>Integer Value that is less than or equal to the specified number.</returns>
		public static long Floor64 (decimal x)
		{
			return (long) System.Decimal.Floor (x);
		}

		#endregion //Floor Methods

		#region Ceiling Methods

		/// <overloads>
		/// <para>Returns the Integer Value that is greater than or equal to the specified number.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// <para>Returns the Integer Value that is greater than or equal to the specified number.</para>
		/// <para>For "larger" values <see cref="Ceiling64"/></para>
		/// </summary>
		/// <param name="x">Value to be processed</param>
		/// <returns>Integer Value that is greater than or equal to the specified number.</returns>
		public static int Ceiling (double x)
		{
			return (int) System.Math.Ceiling (x);
		}

		/// <summary>
		/// <para>Returns the Integer Value that is greater than or equal to the specified number.</para>
		/// <para>For "larger" values <see cref="Ceiling64"/></para>
		/// </summary>
		/// <param name="x">Value to be processed</param>
		/// <returns>Integer Value that is greater than or equal to the specified number.</returns>
		public static int Ceiling (decimal x)
		{
			return (int) DecimalOps.Ceiling (x);
		}

		/// <overloads>
		/// <para>Returns the Integer Value that is greater than or equal to the specified number.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// Returns the Integer Value that is greater than or equal to the specified number.
		/// </summary>
		/// <param name="x">Value to be processed</param>
		/// <returns>Integer Value that is greater than or equal to the specified number.</returns>
		public static long Ceiling64 (double x)
		{
			return (long) System.Math.Ceiling (x);
		}

		/// <summary>
		/// Returns the Integer Value that is greater than or equal to the specified number.
		/// </summary>
		/// <param name="x">Value to be processed</param>
		/// <returns>Integer Value that is greater than or equal to the specified number.</returns>
		public static long Ceiling64 (decimal x)
		{
			return (long) DecimalOps.Ceiling (x);
		}

		#endregion //Ceiling Methods

		#region IntPart Methods

		/// <overloads>
		/// <para>Returns the Integral portion of the value supplied.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// Returns the Integral portion of the value supplied.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Integral portion of the value supplied</returns>
		public static int IntPart (double x)
		{
			if (x >= 0)
			{
				return Floor (x);
			}
			else
			{
				return Ceiling (x);
			}
		}

		/// <summary>
		/// Returns the Integral portion of the value supplied.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Integral portion of the value supplied</returns>
		public static int IntPart (decimal x)
		{
			if (x >= 0)
			{
				return Floor (x);
			}
			else
			{
				return Ceiling (x);
			}
		}

		/// <overloads>
		/// <para>Returns the Integral portion of the value supplied.</para>
		/// <para>Methods for decimal and double parameters included.</para>
		/// </overloads>
		/// <summary>
		/// Returns the Integral portion of the value supplied.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Integral portion of the value supplied</returns>
		public static long IntPart64 (double x)
		{
			if (x >= 0)
			{
				return Floor64 (x);
			}
			else
			{
				return Ceiling64 (x);
			}
		}

		/// <summary>
		/// Returns the Integral portion of the value supplied.
		/// </summary>
		/// <param name="x">Number to be processed.</param>
		/// <returns>Integral portion of the value supplied</returns>
		public static long IntPart64 (decimal x)
		{
			if (x >= 0)
			{
				return Floor64 (x);
			}
			else
			{
				return Ceiling64 (x);
			}
		}
	}
	#endregion //IntPart Methods

}
