using System;

using csUnit;
using Esb.Globals;
using Esb.Math;

namespace EsbMathTests
{
	/// <summary>
	/// Tests for the ESBPCS Decimal Methods
	/// </summary>
	[TestFixture]
	public class DecimalTest
	{
		public DecimalTest()
		{
		}

		[Test]
		public void Sqrt_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(1.0m), 1.0m));
		}
		[Test]

		public void Sqrt_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(2.0m), DecimalGlobals.Sqrt2));
		}
		[Test]
		public void Sqrt_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(3.0m), DecimalGlobals.Sqrt3));
		}
		[Test]
		public void Sqrt_4()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(5.0m), DecimalGlobals.Sqrt5));
		}
		[Test]
		public void Sqrt_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(10.0m), DecimalGlobals.Sqrt10));
		}
		[Test]
		public void Sqrt_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(DecimalGlobals.PI), DecimalGlobals.SqrtPI));
		}
		[Test]
		public void Sqrt_7()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Sqrt(0.0m)));
		}
		[Test]
		public void Sqrt_8()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(0.04m), 0.2m));
		}
		[Test]
		public void Sqrt_9()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(123.4567890m * 123.4567890m), 123.4567890m));
		}
		[Test]
		public void Sqrt_10()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(2.0), DecimalGlobals.Sqrt2));
		}
		[Test]
		public void Sqrt_11()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Sqrt(2), DecimalGlobals.Sqrt2));
		}

		[Test]
		public void IntPow_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.IntPow (0.0m, 55)));
		}
		[Test]
		public void IntPow_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (1.0m, 22), 1.0m));
		}
		[Test]
		public void IntPow_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (1.0m, -22), 1.0m));
		}
		[Test]
		public void IntPow_4()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (0.0m, -55), 0.0m));
		}
		[Test]
		public void IntPow_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (-1.0m, 12), 1.0m));
		}
		[Test]
		public void IntPow_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (-1.0m, 13), -1.0m));
		}
		[Test]
		public void IntPow_7()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (2.0m, 4), 16.0m));
		}
		[Test]
		public void IntPow_8()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (2.0m, -4), 1.0m / 16.0m));
		}
		[Test]
		public void IntPow_9()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (2.5m, 2), 6.25m));
		}
		[Test]
		public void IntPow_10()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (2.5m, -2), 1.0m / 6.25m));
		}
		[Test]
		public void IntPow_11()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (2m, 24), 16777216.0m));
		}
		[Test]
		public void IntPow_12()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (2.0, 4), 16.0m));
		}
		[Test]
		public void IntPow_13()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPow (2, 4), 16.0m));
		}

		[Test]
		public void NthRoot_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.NthRoot (2.0m, 2), DecimalGlobals.Sqrt2));
		}
		[Test]
		public void NthRoot_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.NthRoot (2.0m, 3), DecimalGlobals.Cbrt2));
		}
		[Test]
		public void NthRoot_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.NthRoot (DecimalGlobals.PI, 3), DecimalGlobals.CbrtPI));
		}
		[Test]
		public void NthRoot_4()
		{
			decimal X = DecimalOps.IntPow (0.25m, 12);
			Assert.True (CompareOps.SameValue (DecimalOps.NthRoot (X, 12), 0.25m));
		}
		[Test]
		public void NthRoot_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.NthRoot (2.0, 3), DecimalGlobals.Cbrt2));
		}
		[Test]
		public void NthRoot_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.NthRoot (2, 3), DecimalGlobals.Cbrt2));
		}

		[Test]
		public void Log_1()
		{
			decimal X = DecimalOps.Log (1m);
			Assert.True (CompareOps.SameValue (X, 0.0m));
		}
		[Test]
		public void Log_2()
		{
			decimal X = DecimalOps.Log (DecimalGlobals.NaturalConstant);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Log_3()
		{
			decimal X = DecimalOps.Log (DecimalGlobals.SqrNaturalConstant);
			Assert.True (CompareOps.SameValue (X, 2.0m));
		}
		[Test]
		public void Log_4()
		{
			decimal X = DecimalOps.Log (1.0m / DecimalGlobals.SqrNaturalConstant);
			Assert.True (CompareOps.SameValue (X, -2.0m));
		}
		[Test]
		public void Log_5()
		{
			decimal X = DecimalOps.Log (256.789m);
			Assert.True (CompareOps.SameValue (X, 5.5482547360074002357639148395414m));
		}
		[Test]
		public void Log_6()
		{
			decimal X = DecimalOps.Log (2.0m); 
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.LN2));
		}
		[Test]
		public void Log_7()
		{
			decimal X = DecimalOps.Log (2.0); 
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.LN2));
		}
		[Test]
		public void Log_8()
		{
			decimal X = DecimalOps.Log (2); 
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.LN2));
		}

		[Test]
		public void Log10_1()
		{
			decimal X = DecimalOps.Log10 (1m);
			Assert.True (CompareOps.SameValue (X, 0.0m));
		}
		[Test]
		public void Log10_2()
		{
			decimal X = DecimalOps.Log10 (10.0m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Log10_3()
		{
			decimal X = DecimalOps.Log10 (100.0m);
			Assert.True (CompareOps.SameValue (X, 2.0m));
		}
		[Test]
		public void Log10_4()
		{
			decimal X = DecimalOps.Log10 (0.01m);
			Assert.True (CompareOps.SameValue (X, -2.0m));
		}
		[Test]
		public void Log10_5()
		{
			decimal X = DecimalOps.Log10 (256.789m);
			Assert.True (CompareOps.SameValue (X, 2.4095764160415971281230830903713m));
		}
		[Test]
		public void Log10_6()
		{
			decimal X = DecimalOps.Log10 (2.0m); 
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Log2));
		}
		[Test]
		public void Log10_7()
		{
			decimal X = DecimalOps.Log10 (2.0); 
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Log2));
		}
		[Test]
		public void Log10_8()
		{
			decimal X = DecimalOps.Log10 (2); 
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Log2));
		}

		[Test]
		public void Log2_1()
		{
			decimal X = DecimalOps.Log2 (1m);
			Assert.True (CompareOps.SameValue (X, 0.0m));
		}
		[Test]
		public void Log2_2()
		{
			decimal X = DecimalOps.Log2 (2m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Log2_3()
		{
			decimal X = DecimalOps.Log2 (1024.0m);
			Assert.True (CompareOps.SameValue (X, 10.0m));
		}
		[Test]
		public void Log2_4()
		{
			decimal X = DecimalOps.Log2 (0.0625m);
			Assert.True (CompareOps.SameValue (X, -4.0m));
		}
		[Test]
		public void Log2_5()
		{
			decimal X = DecimalOps.Log2 (256.789m);
			Assert.True (CompareOps.SameValue (X, 8.0044395932265811584534084567608m));
		}
		[Test]
		public void Log2_6()
		{
			decimal X = DecimalOps.Log2 (1024.0);
			Assert.True (CompareOps.SameValue (X, 10.0m));
		}
		[Test]
		public void Log2_7()
		{
			decimal X = DecimalOps.Log2 (1024);
			Assert.True (CompareOps.SameValue (X, 10.0m));
		}

		[Test]
		public void TenPow_1()
		{
			decimal X = DecimalOps.TenPow (0.0m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void TenPow_2()
		{
			decimal X = DecimalOps.TenPow (1.0m);
			Assert.True (CompareOps.SameValue (X, 10.0m));
		}
		[Test]
		public void TenPow_3()
		{
			decimal X = DecimalOps.TenPow (-1.0m);
			Assert.True (CompareOps.SameValue (X, 0.1m));
		}
		[Test]
		public void TenPow_4()
		{
			decimal X = DecimalOps.TenPow (2.0m);
			Assert.True (CompareOps.SameValue (X, 100.0m));
		}
		[Test]
		public void TenPow_5()
		{
			decimal X = DecimalOps.TenPow (-2.0m);
			Assert.True (CompareOps.SameValue (X, 0.01m));
		}
		[Test]
		public void TenPow_6()
		{
			decimal X = DecimalOps.TenPow (0.5m);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt10));
		}
		[Test]
		public void TenPow_7()
		{
			decimal X = DecimalOps.TenPow (12.45m);
			Assert.True (CompareOps.SameValue (X, 2818382931264.4538191019236991551m));
		}
		[Test]
		public void TenPow_8()
		{
			decimal X = DecimalOps.TenPow (2.0);
			Assert.True (CompareOps.SameValue (X, 100.0m));
		}
		[Test]
		public void TenPow_9()
		{
			decimal X = DecimalOps.TenPow (2);
			Assert.True (CompareOps.SameValue (X, 100.0m));
		}

		[Test]
		public void TwoPow_1()
		{
			decimal X = DecimalOps.TwoPow (0.0m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void TwoPow_2()
		{
			decimal X = DecimalOps.TwoPow (1.0m);
			Assert.True (CompareOps.SameValue (X, 2.0m));
		}
		[Test]
		public void TwoPow_3()
		{
			decimal X = DecimalOps.TwoPow (-1.0m);
			Assert.True (CompareOps.SameValue (X, 0.5m));
		}
		[Test]
		public void TwoPow_4()
		{
			decimal X = DecimalOps.TwoPow (2.0m);
			Assert.True (CompareOps.SameValue (X, 4.0m));
		}
		[Test]
		public void TwoPow_5()
		{
			decimal X = DecimalOps.TwoPow (-2.0m);
			Assert.True (CompareOps.SameValue (X, 0.25m));
		}
		[Test]
		public void TwoPow_6()
		{
			decimal X = DecimalOps.TwoPow (0.5m);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt2));
		}
		[Test]
		public void TwoPow_7()
		{
			decimal X = DecimalOps.TwoPow (12.45m);
			Assert.True (CompareOps.SameValue (X, 5595.3008916660040452146378149482m));
		}
		[Test]
		public void TwoPow_8()
		{
			decimal X = DecimalOps.TwoPow (2.0);
			Assert.True (CompareOps.SameValue (X, 4.0m));
		}
		[Test]
		public void TwoPow_9()
		{
			decimal X = DecimalOps.TwoPow (2);
			Assert.True (CompareOps.SameValue (X, 4.0m));
		}



		[Test]
		public void Exp_1()
		{
			decimal X = DecimalOps.Exp (0.0m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Exp_2()
		{
			decimal X = DecimalOps.Exp (1.0m);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.NaturalConstant));
		}
		[Test]
		public void Exp_3()
		{
			decimal X = DecimalOps.Exp (-1.0m);
			Assert.True (CompareOps.SameValue (X, 1 / DecimalGlobals.NaturalConstant));
		}
		[Test]
		public void Exp_4()
		{
			decimal X = DecimalOps.Exp (2.0m);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.SqrNaturalConstant));
		}
		[Test]
		public void Exp_5()
		{
			decimal X = DecimalOps.Exp (-2.0m);
			Assert.True (CompareOps.SameValue (X, 1 / DecimalGlobals.SqrNaturalConstant));
		}
		[Test]
		public void Exp_6()
		{
			decimal X = DecimalOps.Exp (0.5m);
			Assert.True (CompareOps.SameValue (X, DecimalOps.Sqrt (DecimalGlobals.NaturalConstant)));
		}
		[Test]
		public void Exp_7()
		{
			decimal X = DecimalOps.Exp (12.45m);
			Assert.True (CompareOps.SameValue (X, 255250.32262933461218960648692161m));
		}
		[Test]
		public void Exp_8()
		{
			decimal X = DecimalOps.Exp (2.0);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.SqrNaturalConstant));
		}
		[Test]
		public void Exp_9()
		{
			decimal X = DecimalOps.Exp (2);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.SqrNaturalConstant));
		}

		[Test]
		public void Pow_1()
		{
			decimal X = DecimalOps.Pow (4.5m, 0.0m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Pow_2()
		{
			decimal X = DecimalOps.Pow (1.2345678901234567890m, 1.0m);
			Assert.True (CompareOps.SameValue (X, 1.2345678901234567890m));
		}
		[Test]
		public void Pow_3()
		{
			decimal X = DecimalOps.Pow (1.2345678901234567890m, -1.0m);
			Assert.True (CompareOps.SameValue (X, 1 / 1.2345678901234567890m));
		}
		[Test]
		public void Pow_4()
		{
			decimal X = DecimalOps.Pow (1.2345678901234567890m, 0.5m);
			Assert.True (CompareOps.SameValue (X, DecimalOps.Sqrt (1.2345678901234567890m)));
		}
		[Test]
		public void Pow_5()
		{
			decimal X = DecimalOps.Pow (2.0m, 10.0m);
			Assert.True (CompareOps.SameValue (X, 1024.0m));
		}
		[Test]
		public void Pow_6()
		{
			decimal X = DecimalOps.Pow (1024.0m, 0.1m);
			Assert.True (CompareOps.SameValue (X, 2.0m));
		}
		[Test]
		public void Pow_7()
		{
			decimal X = DecimalOps.Pow (3.0m, 3.0m);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}
		[Test]
		public void Pow_8()
		{
			decimal X = DecimalOps.Pow (3.0, 3.0);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}
		[Test]
		public void Pow_9()
		{
			decimal X = DecimalOps.Pow (3, 3);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}
		[Test]
		public void Pow_10()
		{
			decimal X = DecimalOps.Pow (3.0m, 3.0);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}
		[Test]
		public void Pow_11()
		{
			decimal X = DecimalOps.Pow (3.0, 3.0m);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}
		[Test]
		public void Pow_12()
		{
			decimal X = DecimalOps.Pow (3.0m, 3);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}
		[Test]
		public void Pow_13()
		{
			decimal X = DecimalOps.Pow (3, 3.0m);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}
		[Test]
		public void Pow_14()
		{
			decimal X = DecimalOps.Pow (3.0, 3);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}
		[Test]
		public void Pow_15()
		{
			decimal X = DecimalOps.Pow (3, 3.0);
			Assert.True (CompareOps.SameValue (X, 27.0m));
		}

		[Test]
		public void Sin_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Sin (0.0m)));
		}
		[Test]
		public void Sin_2()
		{
			decimal X = DecimalOps.Sin (DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void Sin_3()
		{
			decimal X = DecimalOps.Sin (DecimalGlobals.PIOver2);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Sin_4()
		{
			decimal X = DecimalOps.Sin (-DecimalGlobals.PIOver2);
			Assert.True (CompareOps.SameValue (X, -1.0m));
		}
		[Test]
		public void Sin_5()
		{
			decimal X = DecimalOps.Sin (DecimalGlobals.PI);
			Assert.True (CompareOps.SameValue (X, 0.0m));
		}
		[Test]
		public void Sin_6()
		{
			decimal X = DecimalOps.Sin (-DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, -DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void Sin_7()
		{
			decimal X = DecimalOps.Sin (DecimalGlobals.ThreePIOver4);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void Sin_8()
		{
			decimal X = DecimalOps.Sin (DecimalGlobals.PI + DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, -DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void Sin_9()
		{
			decimal X = DecimalOps.Sin (DecimalGlobals.PIOver3);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt3 / 2.0m));
		}
		[Test]
		public void Sin_10()
		{
			decimal X = DecimalOps.Sin (DecimalGlobals.PIOver6);
			Assert.True (CompareOps.SameValue (X, 0.5m));
		}
		[Test]
		public void Sin_11()
		{
			decimal X = DecimalOps.Sin (1.0m);
			Assert.True (CompareOps.SameValue (X, 0.8414709848078965066525023216303m));
		}
		[Test]
		public void Sin_12()
		{
			decimal X = DecimalOps.Sin (1.0);
			Assert.True (CompareOps.SameValue (X, 0.8414709848078965066525023216303m));
		}

		[Test]
		public void Cos_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Cos (DecimalGlobals.PIOver2)));
		}
		[Test]
		public void Cos_2()
		{
			decimal X = DecimalOps.Cos (DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void Cos_3()
		{
			decimal X = DecimalOps.Cos (0.0m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Cos_4()
		{
			decimal X = DecimalOps.Cos (-DecimalGlobals.PIOver2);
			Assert.True (CompareOps.SameValue (X, 0.0m));
		}
		[Test]
		public void Cos_5()
		{
			decimal X = DecimalOps.Cos (DecimalGlobals.PI);
			Assert.True (CompareOps.SameValue (X, -1.0m));
		}
		[Test]
		public void Cos_6()
		{
			decimal X = DecimalOps.Cos (-DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void Cos_7()
		{
			decimal X = DecimalOps.Cos (DecimalGlobals.ThreePIOver4);
			Assert.True (CompareOps.SameValue (X, -DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void Cos_8()
		{
			decimal X = DecimalOps.Cos (DecimalGlobals.PI + DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, -DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void Cos_9()
		{
			decimal X = DecimalOps.Cos (DecimalGlobals.PIOver3);
			Assert.True (CompareOps.SameValue (X, 0.5m));
		}
		[Test]
		public void Cos_10()
		{
			decimal X = DecimalOps.Cos (DecimalGlobals.PIOver6);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt3 / 2.0m));
		}
		[Test]
		public void Cos_11()
		{
			decimal X = DecimalOps.Cos (1.0m);
			Assert.True (CompareOps.SameValue (X, 0.54030230586813971740093660744298m));
		}
		[Test]
		public void Cos_12()
		{
			decimal X = DecimalOps.Cos (1.0);
			Assert.True (CompareOps.SameValue (X, 0.54030230586813971740093660744298m));
		}

		[Test]
		public void SinCos_1()
		{
			decimal SinX;
			decimal CosX;
			DecimalOps.SinCos (0.0m, out SinX, out CosX);
			Assert.True (CompareOps.SameValue (SinX, 0.0m));
			Assert.True (CompareOps.SameValue (CosX, 1.0m));
		}
		[Test]
		public void SinCos_2()
		{
			decimal SinX;
			decimal CosX;
			DecimalOps.SinCos (DecimalGlobals.PIOver2, out SinX, out CosX);
			Assert.True (CompareOps.SameValue (SinX, 1.0m));
			Assert.True (CompareOps.SameValue (CosX, 0.0m));
		}
		[Test]
		public void SinCos_3()
		{
			decimal SinX;
			decimal CosX;
			DecimalOps.SinCos (DecimalGlobals.PIOver4, out SinX, out CosX);
			Assert.True (CompareOps.SameValue (SinX, DecimalGlobals.Sqrt2 / 2.0m));
			Assert.True (CompareOps.SameValue (CosX, DecimalGlobals.Sqrt2 / 2.0m));
		}
		[Test]
		public void SinCos_4()
		{
			decimal SinX;
			decimal CosX;
			DecimalOps.SinCos (1.0m, out SinX, out CosX);
			Assert.True (CompareOps.SameValue (SinX, 0.8414709848078965066525023216303m));
			Assert.True (CompareOps.SameValue (CosX, 0.54030230586813971740093660744298m));
		}
		[Test]
		public void SinCos_5()
		{
			decimal SinX;
			decimal CosX;
			DecimalOps.SinCos (1.0, out SinX, out CosX);
			Assert.True (CompareOps.SameValue (SinX, 0.8414709848078965066525023216303m));
			Assert.True (CompareOps.SameValue (CosX, 0.54030230586813971740093660744298m));
		}

		[Test]
		public void Tan_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Tan (0.0m)));
		}
		[Test]
		public void Tan_2()
		{
			decimal X = DecimalOps.Tan (DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Tan_3()
		{
			try
			{
				decimal X = DecimalOps.Tan (DecimalGlobals.PIOver2);
			}
			catch (ArithmeticException e)
			{
				e.ToString();
			}
			catch
			{
				Assert.Fail ("Should have thrown an ArithmeticException");
			}
		}
		[Test]
		public void Tan_4()
		{
			try
			{
				decimal X = DecimalOps.Tan (DecimalGlobals.ThreePIOver2);
			}
			catch (ArithmeticException e)
			{
				e.ToString();
			}
			catch
			{
				Assert.Fail ("Should have thrown an ArithmeticException");
			}
		}
		[Test]
		public void Tan_5()
		{
			decimal X = DecimalOps.Tan (DecimalGlobals.PIOver6);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt3 / 3.0m));
		}
		[Test]
		public void Tan_6()
		{
			decimal X = DecimalOps.Tan (DecimalGlobals.PIOver3);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt3));
		}
		[Test]
		public void Tan_7()
		{
			decimal X = DecimalOps.Tan (-DecimalGlobals.PIOver3);
			Assert.True (CompareOps.SameValue (X, -DecimalGlobals.Sqrt3));
		}
		[Test]
		public void Tan_8()
		{
			decimal X = DecimalOps.Tan (DecimalGlobals.PI - DecimalGlobals.PIOver3);
			Assert.True (CompareOps.SameValue (X, -DecimalGlobals.Sqrt3));
		}
		[Test]
		public void Tan_9()
		{
			decimal X = DecimalOps.Tan (DecimalGlobals.PI + DecimalGlobals.PIOver3);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.Sqrt3));
		}
		[Test]
		public void Tan_10()
		{
			decimal X = DecimalOps.Tan (1.0m);
			Assert.True (CompareOps.SameValue (X, 1.5574077246549022305069748074584m));
		}
		[Test]
		public void Tan_11()
		{
			decimal X = DecimalOps.Tan (1.0);
			Assert.True (CompareOps.SameValue (X, 1.5574077246549022305069748074584m));
		}

		[Test]
		public void Deg2Rad_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Rad(180.0m), DecimalGlobals.PI));
		}
		[Test]
		public void Deg2Rad_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Rad(45.0m), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Deg2Rad_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Rad(180.0), DecimalGlobals.PI));
		}
		[Test]
		public void Deg2Rad_4()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Rad(45.0), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Deg2Rad_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Rad(180), DecimalGlobals.PI));
		}
		[Test]
		public void Deg2Rad_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Rad(45), DecimalGlobals.PIOver4));
		}

		[Test]
		public void Rad2Deg_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Rad2Deg(DecimalGlobals.PI), 180.0m));
		}
		[Test]
		public void Rad2Deg_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Rad2Deg(DecimalGlobals.PIOver4), 45.0m));
		}

		[Test]
		public void TanDeg_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.TanDeg (45.0m), 1.0m));
		}
		[Test]
		public void TanDeg_2()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.TanDeg (0.0m)));
		}

		[Test]
		public void Asin_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Asin (0.0m)));
		}
		[Test]
		public void Asin_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Asin (1.0m), DecimalGlobals.PIOver2));
		}
		[Test]
		public void Asin_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Asin (-1.0m), -DecimalGlobals.PIOver2));
		}
		[Test]
		public void Asin_4()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Asin (DecimalGlobals.Sqrt2 / 2.0m), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Asin_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Asin (-DecimalGlobals.Sqrt2 / 2.0m), -DecimalGlobals.PIOver4));
		}
		[Test]
		public void Asin_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Asin (0.5m), DecimalGlobals.PIOver6));
		}
		[Test]
		public void Asin_7()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Asin (-0.5m), -DecimalGlobals.PIOver6));
		}

		[Test]
		public void Acos_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Acos (1.0m)));
		}
		[Test]
		public void Acos_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Acos (-1.0m), DecimalGlobals.PI));
		}
		[Test]
		public void Acos_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Acos (0.0m), DecimalGlobals.PIOver2));
		}
		[Test]
		public void Acos_4()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Acos (DecimalGlobals.Sqrt2 / 2.0m), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Acos_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Acos (-DecimalGlobals.Sqrt2 / 2.0m), DecimalGlobals.ThreePIOver4));
		}
		[Test]
		public void Acos_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Acos (0.5m), DecimalGlobals.PIOver3));
		}
		[Test]
		public void Acos_7()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Acos (-0.5m), DecimalGlobals.PI - DecimalGlobals.PIOver3));
		}

		[Test]
		public void Atan_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Atan (0.0m)));
		}
		[Test]
		public void Atan_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan (1.0m), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Atan_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan (-1.0m), -DecimalGlobals.PIOver4));
		}
		[Test]
		public void Atan_4()
		{
			decimal X = DecimalOps.Atan (DecimalGlobals.Sqrt3);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.PIOver3));
		}
		[Test]
		public void Atan_5()
		{
			decimal X = DecimalOps.Atan (-DecimalGlobals.Sqrt3);
			Assert.True (CompareOps.SameValue (X, -DecimalGlobals.PIOver3));
		}
		[Test]
		public void Atan_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan (DecimalGlobals.Sqrt3 / 3.0m), DecimalGlobals.PIOver6));
		}
		[Test]
		public void Atan_7()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan (-DecimalGlobals.Sqrt3 / 3.0m), -DecimalGlobals.PIOver6));
		}
		[Test]
		public void Atan_8()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan (4.2m), 1.3370531459259951942086396204366m));
		}
		[Test]
		public void Atan_9()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan (-4.2m), -1.3370531459259951942086396204366m));
		}
		[Test]
		public void Atan_10()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan (0.5m), 0.46364760900080611621425623146121m));
		}
		[Test]
		public void Atan_11()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan (-0.5m), -0.46364760900080611621425623146121m));
		}

		[Test]
		public void Atan2_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Atan2 (0.0m, 0.0m)));
		}
		[Test]
		public void Atan2_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (4.5m, 0.0m), DecimalGlobals.PIOver2));
		}
		[Test]
		public void Atan2_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (-4.5m, 0.0m), -DecimalGlobals.PIOver2));
		}
		[Test]
		public void Atan2_4()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Atan2 (0.0m, 4.5m)));
		}
		[Test]
		public void Atan2_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (0.0m, -4.5m), DecimalGlobals.PI));
		}
		[Test]
		public void Atan2_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (2.2m, 2.2m), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Atan2_7()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (-2.2m, 2.2m), -DecimalGlobals.PIOver4));
		}
		[Test]
		public void Atan2_8()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (2.2m, -2.2m), DecimalGlobals.ThreePIOver4));
		}
		[Test]
		public void Atan2_9()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (-2.2m, -2.2m), -DecimalGlobals.ThreePIOver4));
		}
		[Test]
		public void Atan2_10()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (DecimalGlobals.Sqrt3, 3.0m), DecimalGlobals.PIOver6));
		}
		[Test]
		public void Atan2_11()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (-DecimalGlobals.Sqrt3, 3.0m), -DecimalGlobals.PIOver6));
		}
		[Test]
		public void Atan2_12()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (DecimalGlobals.Sqrt3, -3.0m), DecimalGlobals.PI - DecimalGlobals.PIOver6));
		}
		[Test]
		public void Atan2_13()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2 (-DecimalGlobals.Sqrt3, -3.0m), -DecimalGlobals.PI + DecimalGlobals.PIOver6));
		}

		[Test]
		public void AngleAdjustRad_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AdjustAngleRad (2.3m), 2.3m));
		}
		[Test]
		public void AngleAdjustRad_2()
		{
			decimal X = DecimalOps.AdjustAngleRad (-2.3m);
			Assert.True (CompareOps.SameValue (X, -2.3m));
		}
		[Test]
		public void AngleAdjustRad_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AdjustAngleRad (DecimalGlobals.PI), DecimalGlobals.PI));
		}
		[Test]
		public void AngleAdjustRad_4()
		{
			decimal X = DecimalOps.AdjustAngleRad (-DecimalGlobals.PI);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.PI));
		}
		[Test]
		public void AngleAdjustRad_5()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AdjustAngleRad (0.0m)));
		}
		[Test]
		public void AngleAdjustRad_6()
		{
			decimal X = DecimalOps.AdjustAngleRad (12.0m * DecimalGlobals.TwoPI + 1.2m);
			Assert.True (CompareOps.SameValue (X, 1.2m));
		}
		[Test]
		public void AngleAdjustRad_7()
		{
			decimal X = DecimalOps.AdjustAngleRad (12.0m * DecimalGlobals.TwoPI - 1.2m);
			Assert.True (CompareOps.SameValue (X, -1.2m));
		}
		[Test]
		public void AngleAdjustRad_8()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AdjustAngleRad (2.3m, true), 2.3m));
		}
		[Test]
		public void AngleAdjustRad_9()
		{
			decimal X = DecimalOps.AdjustAngleRad (-2.3m, true);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.TwoPI - 2.3m));
		}
		[Test]
		public void AngleAdjustRad_10()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AdjustAngleRad (DecimalGlobals.PI, true), DecimalGlobals.PI));
		}
		[Test]
		public void AngleAdjustRad_11()
		{
			decimal X = DecimalOps.AdjustAngleRad (-DecimalGlobals.PI, true);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.PI));
		}
		[Test]
		public void AngleAdjustRad_12()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AdjustAngleRad (0, true)));
		}
		[Test]
		public void AngleAdjustRad_13()
		{
			decimal X = DecimalOps.AdjustAngleRad (DecimalGlobals.TwoPI, true);
			Assert.True (CompareOps.ValueIsZero (X));
		}
		[Test]
		public void AngleAdjustRad_14()
		{
			decimal X = DecimalOps.AdjustAngleRad (12.0m * DecimalGlobals.TwoPI + 1.2m, true);
			Assert.True (CompareOps.SameValue (X, 1.2m));
		}
		[Test]
		public void AngleAdjustRad_15()
		{
			decimal X = DecimalOps.AdjustAngleRad (12.0m * DecimalGlobals.TwoPI - 1.2m, true);
			Assert.True (CompareOps.SameValue (X, DecimalGlobals.TwoPI - 1.2m));
		}

		[Test]
		public void AngleAdjustDeg_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AdjustAngleDeg (2.3m), 2.3m));
		}
		[Test]
		public void AngleAdjustDeg_2()
		{
			decimal X = DecimalOps.AdjustAngleDeg (-2.3m);
			Assert.True (CompareOps.SameValue (X, -2.3m));
		}
		[Test]
		public void AngleAdjustDeg_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AdjustAngleDeg (180.0m), 180.0m));
		}
		[Test]
		public void AngleAdjustDeg_4()
		{
			decimal X = DecimalOps.AdjustAngleDeg (-180.0m);
			Assert.True (CompareOps.SameValue (X, 180.0m));
		}
		[Test]
		public void AngleAdjustDeg_5()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AdjustAngleDeg (0.0m)));
		}
		[Test]
		public void AngleAdjustDeg_6()
		{
			decimal X = DecimalOps.AdjustAngleDeg (12.0m * 360.0m + 1.2m);
			Assert.True (CompareOps.SameValue (X, 1.2m));
		}
		[Test]
		public void AngleAdjustDeg_7()
		{
			decimal X = DecimalOps.AdjustAngleDeg (12.0m * 360.0m - 1.2m);
			Assert.True (CompareOps.SameValue (X, -1.2m));
		}
		[Test]
		public void AngleAdjustDeg_8()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AdjustAngleDeg (2.3m, true), 2.3m));
		}
		[Test]
		public void AngleAdjustDeg_9()
		{
			decimal X = DecimalOps.AdjustAngleDeg (-2.3m, true);
			Assert.True (CompareOps.SameValue (X, 360.0m - 2.3m));
		}
		[Test]
		public void AngleAdjustDeg_10()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AdjustAngleDeg (180.0m, true), 180.0m));
		}
		[Test]
		public void AngleAdjustDeg_11()
		{
			decimal X = DecimalOps.AdjustAngleDeg (-180.0m, true);
			Assert.True (CompareOps.SameValue (X, 180.0m));
		}
		[Test]
		public void AngleAdjustDeg_12()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AdjustAngleDeg (0.0m, true)));
		}
		[Test]
		public void AngleAdjustDeg_13()
		{
			decimal X = DecimalOps.AdjustAngleDeg (360.0m, true);
			Assert.True (CompareOps.ValueIsZero (X));
		}
		[Test]
		public void AngleAdjustDeg_14()
		{
			decimal X = DecimalOps.AdjustAngleDeg (12.0m * 360.0m + 1.2m, true);
			Assert.True (CompareOps.SameValue (X, 1.2m));
		}
		[Test]
		public void AngleAdjustDeg_15()
		{
			decimal X = DecimalOps.AdjustAngleDeg (12.0m * 360.0m - 1.2m, true);
			Assert.True (CompareOps.SameValue (X, 360m - 1.2m));
		}

		[Test]
		public void Dms2Deg_1()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Dms2Deg (0, 0, 0)));
		}
		[Test]
		public void Dms2Deg_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Dms2Deg (10, 30, 0), 10.5m));
		}
		[Test]
		public void Dms2Deg_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Dms2Deg (10, 30, 36), 10.51m));
		}
		[Test]
		public void Dms2Deg_4()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Dms2Deg (0.0, 0.0, 0.0)));
		}
		[Test]
		public void Dms2Deg_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Dms2Deg (10.0, 30.0, 0.0), 10.5m));
		}
		[Test]
		public void Dms2Deg_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Dms2Deg (10.0, 30.0, 36.0), 10.51m));
		}
		[Test]
		public void Dms2Deg_7()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Dms2Deg (0.0m, 0.0m, 0.0m)));
		}
		[Test]
		public void Dms2Deg_8()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Dms2Deg (10.0m, 30.0m, 0.0m), 10.5m));
		}
		[Test]
		public void Dms2Deg_9()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Dms2Deg (10.0m, 30.0m, 36.0m), 10.51m));
		}

		[Test]
		public void SignXY_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.SignXY (-4.0m, 5.0m), 4.0m));
		}
		[Test]
		public void SignXY_2()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.SignXY (-4.0m, -5.0m), -4.0m));
		}
		[Test]
		public void SignXY_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.SignXY (4.0m, 5.0m), 4.0m));
		}
		[Test]
		public void SignXY_4()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.SignXY (4.0m, -5.0m), -4.0m));
		}

		[Test]
		public void Ceiling_1 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Ceiling (0m)));
		}
		[Test]
		public void Ceiling_2 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Ceiling (-0.123m)));
		}
		[Test]
		public void Ceiling_3 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Ceiling (123.0m), 123.0m));
		}
		[Test]
		public void Ceiling_4 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Ceiling (-123.0m), -123.0m));
		}
		[Test]
		public void Ceiling_5 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Ceiling (123.456m), 124.0m));
		}
		[Test]
		public void Ceiling_6 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Ceiling (-123.456m), -123.0m));
		}

		[Test]
		public void IntPart_1 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.IntPart (0.123m)));
		}
		[Test]
		public void IntPart_2 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.IntPart (-0.123m)));
		}
		[Test]
		public void IntPart_3 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPart (123.456m), 123.0m));
		}
		[Test]
		public void IntPart_4 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPart (-123.456m), -123.0m));
		}
		[Test]
		public void IntPart_5 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.IntPart (0.123)));
		}
		[Test]
		public void IntPart_6 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.IntPart (-0.123)));
		}
		[Test]
		public void IntPart_7 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPart (123.456), 123.0m));
		}
		[Test]
		public void IntPart_8 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.IntPart (-123.456), -123.0m));
		}
	
		[Test]
		public void FractionPart_1 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.FractionPart (123.0m)));
		}
		[Test]
		public void FractionPart_2 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.FractionPart (-123.0m)));
		}
		[Test]
		public void FractionPart_3 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.FractionPart (123.456m), 0.456m));
		}
		[Test]
		public void FractionPart_4 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.FractionPart (-123.456m), -0.456m));
		}
		[Test]
		public void FractionPart_5 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.FractionPart (123.0)));
		}
		[Test]
		public void FractionPart_6 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.FractionPart (-123.0)));
		}
		[Test]
		public void FractionPart_7 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.FractionPart (123.456), 0.456m));
		}
		[Test]
		public void FractionPart_8 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.FractionPart (-123.456), -0.456m));
		}

		[Test]
		public void Deg2DMS_1 ()
		{
			int Degrees;
			int Minutes;
			decimal Seconds;
			int Sign;
			DecimalOps.Deg2Dms (0.0m, out Degrees, out Minutes, out Seconds, out Sign);
			Assert.Equals (Sign, 0);
			Assert.Equals (Degrees, 0);
			Assert.Equals (Minutes, 0);
			Assert.True (CompareOps.ValueIsZero (Seconds));
		}
		[Test]
		public void Deg2DMS_2 ()
		{
			int Degrees;
			int Minutes;
			decimal Seconds;
			int Sign;
			DecimalOps.Deg2Dms (12.0m, out Degrees, out Minutes, out Seconds, out Sign);
			Assert.Equals (Sign, 1);
			Assert.Equals (Degrees, 12);
			Assert.Equals (Minutes, 0);
			Assert.True (CompareOps.ValueIsZero (Seconds));
		}
		[Test]
		public void Deg2DMS_3 ()
		{
			int Degrees;
			int Minutes;
			decimal Seconds;
			int Sign;
			DecimalOps.Deg2Dms (-12.0, out Degrees, out Minutes, out Seconds, out Sign);
			Assert.Equals (Sign, -1);
			Assert.Equals (Degrees, 12);
			Assert.Equals (Minutes, 0);
			Assert.True (CompareOps.ValueIsZero (Seconds));
		}
		[Test]
		public void Deg2DMS_4 ()
		{
			int Degrees;
			int Minutes;
			decimal Seconds;
			int Sign;
			DecimalOps.Deg2Dms (12.5m, out Degrees, out Minutes, out Seconds, out Sign);
			Assert.Equals (Sign, 1);
			Assert.Equals (Degrees, 12);
			Assert.Equals (Minutes, 30);
			Assert.True (CompareOps.ValueIsZero (Seconds));
		}
		[Test]
		public void Deg2DMS_5 ()
		{
			int Degrees;
			int Minutes;
			decimal Seconds;
			int Sign;
			DecimalOps.Deg2Dms (-12.5, out Degrees, out Minutes, out Seconds, out Sign);
			Assert.Equals (Sign, -1);
			Assert.Equals (Degrees, 12);
			Assert.Equals (Minutes, 30);
			Assert.True (CompareOps.ValueIsZero (Seconds));
		}
		[Test]
		public void Deg2DMS_6 ()
		{
			int Degrees;
			int Minutes;
			decimal Seconds;
			int Sign;
			DecimalOps.Deg2Dms (12.51m, out Degrees, out Minutes, out Seconds, out Sign);
			Assert.Equals (Sign, 1);
			Assert.Equals (Degrees, 12);
			Assert.Equals (Minutes, 30);
			Assert.True (CompareOps.SameValue (Seconds, 36.0m));
		}
		[Test]
		public void Deg2DMS_7 ()
		{
			int Degrees;
			int Minutes;
			decimal Seconds;
			int Sign;
			DecimalOps.Deg2Dms (-12.51, out Degrees, out Minutes, out Seconds, out Sign);
			Assert.Equals (Sign, -1);
			Assert.Equals (Degrees, 12);
			Assert.Equals (Minutes, 30);
			Assert.True (CompareOps.SameValue (Seconds, 36.0m));
		}

		[Test]
		public void Rad2Grad1 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Rad2Grad (0.0m)));
		}
		[Test]
		public void Rad2Grad2 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Rad2Grad (DecimalGlobals.PIOver4), 50.0m));
		}
		[Test]
		public void Rad2Grad3 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Rad2Grad (-DecimalGlobals.PIOver4), -50.0m));
		}
		[Test]
		public void Rad2Grad4 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Rad2Grad (0.0)));
		}
		[Test]
		public void Rad2Grad5 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Rad2Grad (1.5), 95.492965855137201461330258023509m));
		}
		[Test]
		public void Rad2Grad6 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Rad2Grad (-1.5), -95.492965855137201461330258023509m));
		}
		[Test]
		public void Rad2Grad7 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Rad2Grad (0)));
		}

		[Test]
		public void Deg2Grad1 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Deg2Grad (0.0m)));
		}
		[Test]
		public void Deg2Grad2 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Grad (45.0m), 50.0m));
		}
		[Test]
		public void Deg2Grad3 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Grad (-45.0m), -50.0m));
		}
		[Test]
		public void Deg2Grad4 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Deg2Grad (0.0)));
		}
		[Test]
		public void Deg2Grad5 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Grad (45.0), 50.0m));
		}
		[Test]
		public void Deg2Grad6 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Grad (-45.0), -50.0m));
		}
		[Test]
		public void Deg2Grad7 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Deg2Grad (0)));
		}
		[Test]
		public void Deg2Grad8 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Grad (45), 50.0m));
		}
		[Test]
		public void Deg2Grad9 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Deg2Grad (-45), -50.0m));
		}

		[Test]
		public void Grad2Rad1 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Grad2Rad (0.0m)));
		}
		[Test]
		public void Grad2Rad2 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Rad (50.0m), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Grad2Rad3 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Rad (-50.0m), -DecimalGlobals.PIOver4));
		}
		[Test]
		public void Grad2Rad4 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Grad2Rad (0.0)));
		}
		[Test]
		public void Grad2Rad5 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Rad (50.0), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Grad2Rad6 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Rad (-50.0), -DecimalGlobals.PIOver4));
		}
		[Test]
		public void Grad2Rad7 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Grad2Rad (0)));
		}
		[Test]
		public void Grad2Rad8 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Rad (50), DecimalGlobals.PIOver4));
		}
		[Test]
		public void Grad2Rad9 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Rad (-50), -DecimalGlobals.PIOver4));
		}

		[Test]
		public void Grad2Deg1 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Grad2Deg (0.0m)));
		}
		[Test]
		public void Grad2Deg2 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Deg (50.0m), 45.0m));
		}
		[Test]
		public void Grad2Deg3 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Deg (-50.0m), -45.0m));
		}
		[Test]
		public void Grad2Deg4 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Grad2Deg (0.0)));
		}
		[Test]
		public void Grad2Deg5 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Deg (50.0), 45.0m));
		}
		[Test]
		public void Grad2Deg6 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Deg (-50.0), -45.0m));
		}
		[Test]
		public void Grad2Deg7 ()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Grad2Deg (0)));
		}
		[Test]
		public void Grad2Deg8 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Deg (50), 45.0m));
		}
		[Test]
		public void Grad2Deg9 ()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Grad2Deg (-50), -45.0m));
		}

		[Test]
		public void AtanDeg_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AtanDeg (1.0m), 45.0m));
		}
		[Test]
		public void AtanDeg_2()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AtanDeg (0.0m)));
		}
		[Test]
		public void AtanDeg_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AtanDeg (1.0), 45.0m));
		}
		[Test]
		public void AtanDeg_4()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AtanDeg (0.0)));
		}
		[Test]
		public void AtanDeg_5()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AtanDeg (1), 45.0m));
		}
		[Test]
		public void AtanDeg62()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AtanDeg (0)));
		}

		[Test]
		public void Atan2Deg_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2Deg (4.0m, 4.0m), 45.0m));
		}
		[Test]
		public void Atan3Deg_2()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Atan2Deg (0.0m, 5.5m)));
		}
		[Test]
		public void Atan2Deg_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2Deg (-4.0m, 4.0m), -45.0m));
		}
		[Test]
		public void Atan2Deg_4()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2Deg (4.0, 4.0), 45.0m));
		}
		[Test]
		public void Atan3Deg_5()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Atan2Deg (0.0, 5.5)));
		}
		[Test]
		public void Atan2Deg_6()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2Deg (-4.0, 4.0), -45.0m));
		}
		[Test]
		public void Atan2Deg_7()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2Deg (4, 4), 45.0m));
		}
		[Test]
		public void Atan3Deg_8()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.Atan2Deg (0, 5)));
		}
		[Test]
		public void Atan2Deg_9()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.Atan2Deg (-4, 4), -45.0m));
		}

		[Test]
		public void AsinDeg_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AsinDeg (DecimalGlobals.Sqrt2 / 2.0m), 45.0m));
		}
		[Test]
		public void AsinDeg_2()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AsinDeg (0.0m)));
		}
		[Test]
		public void AsinDeg_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AsinDeg (1.0m), 90.0m));
		}
	
		[Test]
		public void AcosDeg_1()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AcosDeg (DecimalGlobals.Sqrt2 / 2.0m), 45.0m));
		}
		[Test]
		public void AcosDeg_2()
		{
			Assert.True (CompareOps.ValueIsZero (DecimalOps.AcosDeg (1.0m)));
		}
		[Test]
		public void AcosDeg_3()
		{
			Assert.True (CompareOps.SameValue (DecimalOps.AcosDeg (0.0m), 90.0m));
		}

		[Test]
		public void Cosec_1()
		{
			try
			{
				Decimal x = DecimalOps.Cosec (0.0m);
				Assert.Fail ("Expected an Exception");
			}
			catch
			{
			}
		}
		[Test]
		public void Cosec_2()
		{
			decimal X = DecimalOps.Cosec (DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, 2.0m / DecimalGlobals.Sqrt2));
		}
		[Test]
		public void Cosec_3()
		{
			decimal X = DecimalOps.Cosec (DecimalGlobals.PIOver2);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}

		[Test]
		public void Sec_1()
		{
			decimal X = DecimalOps.Sec (0.0m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Sec_2()
		{
			decimal X = DecimalOps.Sec (DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, 2.0m / DecimalGlobals.Sqrt2));
		}
		[Test]
		public void Sec_3()
		{
			try
			{
				Decimal x = DecimalOps.Sec (DecimalGlobals.PIOver2);
				Assert.Fail ("Expected an Exception");
			}
			catch
			{
			}
		}

		[Test]
		public void Cot_1()
		{
			try
			{
				Decimal x = DecimalOps.Cot (0.0m);
				Assert.Fail ("Expected an Exception");
			}
			catch
			{
			}
		}
		[Test]
		public void Cot_2()
		{
			decimal X = DecimalOps.Cot (DecimalGlobals.PIOver4);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void Cot_3()
		{
			decimal X = DecimalOps.Cot (DecimalGlobals.PIOver2);
			Assert.True (CompareOps.ValueIsZero (X));
		}

		[Test]
		public void CosecDeg_1()
		{
			try
			{
				Decimal x = DecimalOps.CosecDeg (0.0m);
				Assert.Fail ("Expected an Exception");
			}
			catch
			{
			}
		}
		[Test]
		public void CosecDeg_2()
		{
			decimal X = DecimalOps.CosecDeg (45.0);
			Assert.True (CompareOps.SameValue (X, 2.0m / DecimalGlobals.Sqrt2));
		}
		[Test]
		public void CosecDeg_3()
		{
			decimal X = DecimalOps.CosecDeg (90);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}

		[Test]
		public void SecDeg_1()
		{
			decimal X = DecimalOps.SecDeg (0.0m);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void SecDeg_2()
		{
			decimal X = DecimalOps.SecDeg (45.0);
			Assert.True (CompareOps.SameValue (X, 2.0m / DecimalGlobals.Sqrt2));
		}
		[Test]
		public void SecDeg_3()
		{
			try
			{
				Decimal x = DecimalOps.SecDeg (90);
				Assert.Fail ("Expected an Exception");
			}
			catch
			{
			}
		}

		[Test]
		public void CotDeg_1()
		{
			try
			{
				Decimal x = DecimalOps.CotDeg (0.0m);
				Assert.Fail ("Expected an Exception");
			}
			catch
			{
			}
		}
		[Test]
		public void CotDeg_2()
		{
			decimal X = DecimalOps.CotDeg (45.0);
			Assert.True (CompareOps.SameValue (X, 1.0m));
		}
		[Test]
		public void CotDeg_3()
		{
			decimal X = DecimalOps.CotDeg (90);
			Assert.True (CompareOps.ValueIsZero (X));
		}

	}
}
